@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 
 ## system-config-printer
 
@@ -17,10 +17,10 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import os
-import glib
+from gi.repository import GLib
 
 class PackageKit:
     def __init__ (self):
@@ -30,7 +30,7 @@ class PackageKit:
                 self.gpk_install_package_name = path
                 return
 
-        raise RuntimeError, "No gpk-install-package-name program available"
+        raise RuntimeError("No gpk-install-package-name program available")
 
     def InstallPackageName (self, xid, timestamp, name):
-        glib.spawn_async ([self.gpk_install_package_name, name])
+        GLib.spawn_async ([self.gpk_install_package_name, name])
