@@ -1,5 +1,5 @@
 
-## Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Red Hat, Inc.
 ## Authors:
 ##  Tim Waugh <twaugh@redhat.com>
 ##  Jiri Popelka <jpopelka@redhat.com>
@@ -39,13 +39,14 @@ import smburi
 import subprocess
 import sys
 import time
-import urllib
+import urllib.parse
 from xml.sax import saxutils
 
 from debug import *
 import config
 import statereason
 import errordialogs
+from functools import reduce
 
 cups.require("1.9.47")
 
@@ -56,7 +57,7 @@ except ImportError:
     USE_KEYRING=False
 
 import gettext
-gettext.install(domain=config.PACKAGE, localedir=config.localedir, unicode=True)
+gettext.install(domain=config.PACKAGE, localedir=config.localedir)
 
 from statereason import StateReason
 
@@ -97,14 +98,14 @@ class PrinterURIIndex:
 
     def update_from_attrs (self, printer, attrs):
         uris = []
-        if attrs.has_key ('printer-uri-supported'):
+        if 'printer-uri-supported' in attrs:
             uri_supported = attrs['printer-uri-supported']
             if type (uri_supported) != list:
                 uri_supported = [uri_supported]
             uris.extend (uri_supported)
-        if attrs.has_key ('notify-printer-uri'):
+        if 'notify-printer-uri' in attrs:
             uris.append (attrs['notify-printer-uri'])
-        if attrs.has_key ('printer-more-info'):
+        if 'printer-more-info' in attrs:
             uris.append (attrs['printer-more-info'])
 
         for uri in uris:
@@ -113,7 +114,7 @@ class PrinterURIIndex:
     def remove_printer (self, printer):
         # Remove references to this printer in the URI map.
         self._collect_names ()
-        uris = self.printer.keys ()
+        uris = list(self.printer.keys ())
         for uri in uris:
             if self.printer[uri] == printer:
                 del self.printer[uri]
@@ -131,7 +132,7 @@ class PrinterURIIndex:
 
     def lookup_cached_by_name (self, name):
         self._collect_names ()
-        for uri, printer in self.printer.iteritems ():
+        for uri, printer in self.printer.items ():
             if printer == name:
                 return uri
 
@@ -141,8 +142,6 @@ class PrinterURIIndex:
         try:
             if connection == None:
                 connection = cups.Connection ()
-            if isinstance(name, bytes):
-                name = name.decode ('utf-8')
 
             r = ['printer-name', 'printer-uri-supported', 'printer-more-info']
             if uri != None:
@@ -196,11 +195,10 @@ class CancelJobsOperation(GObject.GObject):
                 dialog_title = _("Cancel Job")
                 dialog_label = _("Do you really want to cancel this job?")
 
-        dialog = Gtk.Dialog (dialog_title, parent,
-                             Gtk.DialogFlags.MODAL |
-                             Gtk.DialogFlags.DESTROY_WITH_PARENT,
-                             (_("Keep Printing"), Gtk.ResponseType.NO,
-                              dialog_title, Gtk.ResponseType.YES))
+        dialog = Gtk.Dialog (title=dialog_title, transient_for=parent,
+                             modal=True, destroy_with_parent=True)
+        dialog.add_buttons (_("Keep Printing"), Gtk.ResponseType.NO,
+                            dialog_title, Gtk.ResponseType.YES)
         dialog.set_default_response (Gtk.ResponseType.NO)
         dialog.set_border_width (6)
         dialog.set_resizable (False)
@@ -363,7 +361,7 @@ class JobViewer (GtkGUI):
 
                          domain=config.PACKAGE)
 
-        job_action_group = Gtk.ActionGroup ("JobActionGroup")
+        job_action_group = Gtk.ActionGroup (name="JobActionGroup")
         job_action_group.add_actions ([
                 ("cancel-job", Gtk.STOCK_CANCEL, _("_Cancel"), None,
                  _("Cancel selected jobs"), self.on_job_cancel_activate),
@@ -589,7 +587,7 @@ class JobViewer (GtkGUI):
         self.JobsAttributesWindow.connect("delete_event",
                                           self.job_attributes_on_delete_event)
         self.JobsAttributesWindow.add_accel_group (self.job_ui_manager.get_accel_group ())
-        attrs_action_group = Gtk.ActionGroup ("AttrsActionGroup")
+        attrs_action_group = Gtk.ActionGroup (name="AttrsActionGroup")
         attrs_action_group.add_actions ([
                 ("close", Gtk.STOCK_CLOSE, None, "<ctrl>w",
                  _("Close this window"), self.job_attributes_on_delete_event)
@@ -716,9 +714,9 @@ class JobViewer (GtkGUI):
     def update_job_creation_times(self):
         now = time.time ()
         need_update = False
-        for job, data in self.jobs.iteritems():
+        for job, data in self.jobs.items():
             t = _("Unknown")
-            if data.has_key ('time-at-creation'):
+            if 'time-at-creation' in data:
                 created = data['time-at-creation']
                 ago = now - created
                 need_update = True
@@ -749,7 +747,7 @@ class JobViewer (GtkGUI):
                     need_update = False
                     t = time.strftime ("%B %Y", time.localtime (created))
 
-            if self.jobiters.has_key (job):
+            if job in self.jobiters:
                 iter = self.jobiters[job]
                 self.store.set_value (iter, 1, t)
 
@@ -777,7 +775,7 @@ class JobViewer (GtkGUI):
         self.stopped_job_prompts.remove (jobid)
         if response == Gtk.ResponseType.NO:
             # Diagnose
-            if not self.__dict__.has_key ('troubleshooter'):
+            if 'troubleshooter' not in self.__dict__:
                 import troubleshoot
                 troubleshooter = troubleshoot.run (self.on_troubleshoot_quit)
                 self.troubleshooter = troubleshooter
@@ -790,7 +788,7 @@ class JobViewer (GtkGUI):
 
         # There may have been an error fetching additional attributes,
         # in which case we need to give up.
-        if not self.jobs.has_key (job):
+        if job not in self.jobs:
             return
 
         store = self.store
@@ -914,14 +912,15 @@ class JobViewer (GtkGUI):
             # running as an applet just try to get the information silently
             # and not prompt the user.
             self.get_authentication (job, data.get ('device-uri'),
+                                     data.get ('job-printer-uri'),
                                      data.get ('auth-info-required', []),
                                      self.applet)
         self.update_sensitivity ()
 
-    def get_authentication (self, job, device_uri, auth_info_required,
-                            show_dialog):
+    def get_authentication (self, job, device_uri, printer_uri,
+                            auth_info_required, show_dialog):
         # Check if we have requested authentication for this job already
-        if not self.auth_info_dialogs.has_key (job):
+        if job not in self.auth_info_dialogs:
             try:
                 cups.require ("1.9.37")
             except:
@@ -931,62 +930,72 @@ class JobViewer (GtkGUI):
 
             # Find out which auth-info is required.
             try_keyring = USE_KEYRING
-            keyring_attrs = dict()
+            informational_attrs = dict()
             auth_info = None
             if try_keyring and 'password' in auth_info_required:
-                (scheme, rest) = urllib.splittype (device_uri)
+                (scheme, rest) = urllib.parse.splittype (device_uri)
                 if scheme == 'smb':
                     uri = smburi.SMBURI (uri=device_uri)
                     (group, server, share,
                      user, password) = uri.separate ()
-                    keyring_attrs["domain"] = str (group)
+                    informational_attrs["domain"] = str (group)
                 else:
-                    (serverport, rest) = urllib.splithost (rest)
+                    (serverport, rest) = urllib.parse.splithost (rest)
                     if serverport == None:
                         server = None
                     else:
-                        (server, port) = urllib.splitnport (serverport)
+                        (server, port) = urllib.parse.splitnport (serverport)
 
                 if scheme == None or server == None:
                     try_keyring = False
                 else:
-                    keyring_attrs.update ({ "server": str (server.lower ()),
-                                            "protocol": str (scheme)})
+                    informational_attrs.update ({ "server": str (server.lower ()),
+                                                 "protocol": str (scheme)})
 
             if job in self.authenticated_jobs:
                 # We've already tried to authenticate this job before.
                 try_keyring = False
 
+            # To increase compatibility and resolve problems with
+            # multiple printers on one host we use the printers URI
+            # as the identifying attribute. Versions <= 1.4.4 used
+            # a combination of domain / server / protocol instead.
+            # The old attributes are still used as a fallback for identifying
+            # the secret but are otherwise only informational.
+            identifying_attrs = { "uri": str (printer_uri) }
+
             if try_keyring and 'password' in auth_info_required:
                 type = GnomeKeyring.ItemType.NETWORK_PASSWORD
-                attrs = GnomeKeyring.Attribute.list_new ()
-                for key, val in keyring_attrs.iteritems ():
-                    GnomeKeyring.Attribute.list_append_string (attrs,
-                                                               key,
-                                                               val)
-                (result, items) = GnomeKeyring.find_items_sync (type,
-                                                                attrs)
-                if result == GnomeKeyring.Result.OK:
-                    auth_info = map (lambda x: '', auth_info_required)
-                    ind = auth_info_required.index ('username')
 
-                    for attr in GnomeKeyring.attribute_list_to_glist (
-                            items[0].attributes):
-                        # It might be safe to assume here that the
-                        # user element is always the second item in a
-                        # NETWORK_PASSWORD element but lets make sure.
-                        if attr.name == 'user':
-                            auth_info[ind] = attr.get_string()
-                            break
-                    else:
-                        debugprint ("Did not find username keyring "
-                                    "attributes.")
+                for keyring_attrs in [identifying_attrs, informational_attrs]:
+                    attrs = GnomeKeyring.Attribute.list_new ()
+                    for key, val in keyring_attrs.items ():
+                        GnomeKeyring.Attribute.list_append_string (attrs,
+                                                                   key,
+                                                                   val)
+                    (result, items) = GnomeKeyring.find_items_sync (type,
+                                                                    attrs)
+                    if result == GnomeKeyring.Result.OK:
+                        auth_info = ['' for x in auth_info_required]
+                        ind = auth_info_required.index ('username')
 
-                    ind = auth_info_required.index ('password')
-                    auth_info[ind] = items[0].secret
+                        for attr in GnomeKeyring.attribute_list_to_glist (
+                                items[0].attributes):
+                            # It might be safe to assume here that the
+                            # user element is always the second item in a
+                            # NETWORK_PASSWORD element but lets make sure.
+                            if attr.name == 'user':
+                                auth_info[ind] = attr.get_string()
+                                break
+                        else:
+                            debugprint ("Did not find username keyring "
+                                        "attributes.")
+
+                        ind = auth_info_required.index ('password')
+                        auth_info[ind] = items[0].secret
+                        break
                 else:
-                    debugprint ("gnomekeyring: look-up result %s" %
-                                repr (result))
+                    debugprint ("Failed to find secret in keyring.")
 
             if try_keyring:
                 try:
@@ -1016,6 +1025,8 @@ class JobViewer (GtkGUI):
 
             if auth_info_required and show_dialog:
                 username = pwd.getpwuid (os.getuid ())[0]
+                keyring_attrs = informational_attrs.copy()
+                keyring_attrs.update(identifying_attrs)
                 keyring_attrs["user"] = str (username)
                 self.display_auth_info_dialog (job, keyring_attrs)
 
@@ -1029,7 +1040,7 @@ class JobViewer (GtkGUI):
         dialog.set_position (Gtk.WindowPosition.CENTER)
 
         # Pre-fill 'username' field.
-        auth_info = map (lambda x: '', auth_info_required)
+        auth_info = ['' for x in auth_info_required]
         username = pwd.getpwuid (os.getuid ())[0]
         if 'username' in auth_info_required:
             try:
@@ -1116,7 +1127,7 @@ class JobViewer (GtkGUI):
                     ind = auth_info_required.index ('password')
                     secret = auth_info[ind]
                     attrs = GnomeKeyring.Attribute.list_new ()
-                    for key, val in keyring_attrs.iteritems ():
+                    for key, val in keyring_attrs.items ():
                         GnomeKeyring.Attribute.list_append_string (attrs,
                                                                    key,
                                                                    val)
@@ -1144,7 +1155,7 @@ class JobViewer (GtkGUI):
 
         open_notifications = len (self.new_printer_notifications.keys ())
         open_notifications += len (self.completed_job_notifications.keys ())
-        for reason, notification in self.state_reason_notifications.iteritems():
+        for reason, notification in self.state_reason_notifications.items():
             if getattr (notification, 'closed', None) != True:
                 open_notifications += 1
         num_jobs = len (self.active_jobs)
@@ -1169,7 +1180,7 @@ class JobViewer (GtkGUI):
             Gtk.main_iteration ()
 
     def on_treeview_popup_menu (self, treeview):
-        event = Gdk.Event (Gdk.NOTHING)
+        event = Gdk.Event (Gdk.EventType.NOTHING)
         self.show_treeview_popup_menu (treeview, event, 0)
 
     def on_treeview_button_release_event(self, treeview, event):
@@ -1211,7 +1222,7 @@ class JobViewer (GtkGUI):
             self.jobids.append(jobid)
             job = self.jobs[jobid]
 
-            if job.has_key ('job-state'):
+            if 'job-state' in job:
                 s = job['job-state']
                 if s >= cups.IPP_JOB_CANCELED:
                     cancel_sensitive = False
@@ -1237,7 +1248,7 @@ class JobViewer (GtkGUI):
 
         if len (job_printers.keys ()) == 1:
             try:
-                other_printers.remove (job_printers.keys ()[0])
+                other_printers.remove (list(job_printers.keys ())[0])
             except KeyError:
                 pass
 
@@ -1286,7 +1297,7 @@ class JobViewer (GtkGUI):
 
     def on_icon_configure_printers_activate(self, menuitem):
         env = {}
-        for name, value in os.environ.iteritems ():
+        for name, value in os.environ.items ():
             if name == "SYSTEM_CONFIG_PRINTER_UI":
                 continue
             env[name] = value
@@ -1419,7 +1430,11 @@ class JobViewer (GtkGUI):
             try:
                 attrs=c.getJobAttributes(jobid)
                 printer_uri=attrs['job-printer-uri']
-                document_count=attrs.get ('document-count', 0)
+                try:
+                    document_count = attrs['number-of-documents']
+                except KeyError:
+                    document_count = attrs.get ('document-count', 0)
+
                 for document_number in range(1, document_count+1):
                     document=c.getDocument(printer_uri, jobid, document_number)
                     tempfile = document.get('file')
@@ -1439,11 +1454,12 @@ class JobViewer (GtkGUI):
                             name = name + format.replace('application/', '.')
 
                     if tempfile != None:
-                        dialog = Gtk.FileChooserDialog (_("Save File"),
-                                                        self.JobsWindow,
-                                                  Gtk.FileChooserAction.SAVE,
-                                        (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
-                                         Gtk.STOCK_SAVE, Gtk.ResponseType.OK))
+                        dialog = Gtk.FileChooserDialog (title=_("Save File"),
+                                            transient_for=self.JobsWindow,
+                                            action=Gtk.FileChooserAction.SAVE)
+                        dialog.add_buttons (
+                                    Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+                                    Gtk.STOCK_SAVE, Gtk.ResponseType.OK)
                         dialog.set_current_name(name)
                         dialog.set_do_overwrite_confirmation(True)
 
@@ -1516,7 +1532,7 @@ class JobViewer (GtkGUI):
                 auth_info_required = ['username', 'password']
 
             self.get_authentication (jobid, pattrs.get ('device-uri'),
-                                     auth_info_required, True)
+                                     uri, auth_info_required, True)
 
     def on_refresh_clicked(self, toolbutton):
         self.monitor.refresh ()
@@ -1583,7 +1599,7 @@ class JobViewer (GtkGUI):
                 return
 
             attr_store.clear()                          # remove old attributes
-            for name, value in attrs.iteritems():
+            for name, value in attrs.items():
                 if name in ['job-id', 'job-printer-up-time']:
                     continue
                 attr_store.append([name, str(value)])
@@ -1648,7 +1664,7 @@ class JobViewer (GtkGUI):
 
         if have_jobs:
             pixbuf = self.icon_jobs
-            for jobid, jobdata in self.jobs.iteritems ():
+            for jobid, jobdata in self.jobs.items ():
                 jstate = jobdata.get ('job-state', cups.IPP_JOB_PENDING)
                 if jstate == cups.IPP_JOB_PROCESSING:
                     pixbuf = self.icon_jobs_processing
@@ -1681,7 +1697,7 @@ class JobViewer (GtkGUI):
     def update_status (self, have_jobs=None):
         # Found out which printer state reasons apply to our active jobs.
         upset_printers = set()
-        for printer, reasons in self.printer_state_reasons.iteritems ():
+        for printer, reasons in self.printer_state_reasons.items ():
             if len (reasons) > 0:
                 upset_printers.add (printer)
         debugprint ("Upset printers: %s" % upset_printers)
@@ -1780,7 +1796,7 @@ class JobViewer (GtkGUI):
 
     def notify_printer_state_reason (self, reason):
         tuple = reason.get_tuple ()
-        if self.state_reason_notifications.has_key (tuple):
+        if tuple in self.state_reason_notifications:
             debugprint ("Already sent notification for %s" % repr (reason))
             return
 
@@ -1840,7 +1856,7 @@ class JobViewer (GtkGUI):
                 device_uri = attrs.get ('device-uri')
 
             if device_uri != None:
-                (scheme, rest) = urllib.splittype (device_uri)
+                (scheme, rest) = urllib.parse.splittype (device_uri)
                 if scheme not in ['socket', 'ipp', 'http', 'smb']:
                     return
 
@@ -1889,17 +1905,17 @@ class JobViewer (GtkGUI):
 
         # We may be showing this job already, perhaps because we are showing
         # completed jobs and one was reprinted.
-        if not self.jobiters.has_key (jobid):
+        if jobid not in self.jobiters:
             self.add_job (jobid, jobdata)
         elif mon == self.my_monitor:
             # Copy over any missing attributes such as user and title.
-            for attr, value in jobdata.iteritems ():
-                if not self.jobs[jobid].has_key (attr):
+            for attr, value in jobdata.items ():
+                if attr not in self.jobs[jobid]:
                     self.jobs[jobid][attr] = value
                     debugprint ("Add %s=%s (my job)" % (attr, value))
 
         # If we failed to get required attributes for the job, bail.
-        if not self.jobiters.has_key (jobid):
+        if jobid not in self.jobiters:
             return
 
         if self.job_is_active (jobdata):
@@ -1941,7 +1957,7 @@ class JobViewer (GtkGUI):
         # that should probably be deferred to the idle handler, but
         # for the moment just deal with the fact that the job might
         # have gone (bug #640904).
-        if not self.jobs.has_key (jobid):
+        if jobid not in self.jobs:
             return
 
         jobdata = self.jobs[jobid]
@@ -2026,9 +2042,10 @@ class JobViewer (GtkGUI):
             if may_be_problem:
                 debugprint ("Problem detected")
                 self.toggle_window_display (None, force_show=True)
-                dialog = Gtk.Dialog (_("Print Error"), self.JobsWindow, 0,
-                                     (_("_Diagnose"), Gtk.ResponseType.NO,
-                                        Gtk.STOCK_OK, Gtk.ResponseType.OK))
+                dialog = Gtk.Dialog (title=_("Print Error"),
+                                     transient_for=self.JobsWindow)
+                dialog.add_buttons (_("_Diagnose"), Gtk.ResponseType.NO,
+                                    Gtk.STOCK_OK, Gtk.ResponseType.OK)
                 dialog.set_default_response (Gtk.ResponseType.OK)
                 dialog.set_border_width (6)
                 dialog.set_resizable (False)
@@ -2084,7 +2101,7 @@ class JobViewer (GtkGUI):
             if not canceled:
                 self.notify_completed_job (jobid)
 
-        if self.jobiters.has_key (jobid):
+        if jobid in self.jobiters:
             self.store.remove (self.jobiters[jobid])
             del self.jobiters[jobid]
             del self.jobs[jobid]
@@ -2092,7 +2109,7 @@ class JobViewer (GtkGUI):
         if jobid in self.active_jobs:
             self.active_jobs.remove (jobid)
 
-        if self.jobs_attrs.has_key (jobid):
+        if jobid in self.jobs_attrs:
             del self.jobs_attrs[jobid]
 
         self.update_status ()
@@ -2116,7 +2133,7 @@ class JobViewer (GtkGUI):
             return
 
         # Find out if the user has jobs queued for that printer.
-        for job, data in self.jobs.iteritems ():
+        for job, data in self.jobs.items ():
             if not self.job_is_active (data):
                 continue
             if data['job-printer-name'] == printer:
@@ -2230,18 +2247,30 @@ class JobViewer (GtkGUI):
 
     def _set_job_user_text (self, column, cell, model, iter, *data):
         jobid = model.get_value (iter, 0)
-        job = self.jobs[jobid]
+        try:
+            job = self.jobs[jobid]
+        except KeyError:
+            return
+
         cell.set_property("text", job.get ('job-originating-user-name',
                                            _("Unknown")))
 
     def _set_job_document_text (self, column, cell, model, iter, *data):
         jobid = model.get_value (iter, 0)
-        job = self.jobs[jobid]
+        try:
+            job = self.jobs[jobid]
+        except KeyError:
+            return
+
         cell.set_property("text", job.get('job-name', _("Unknown")))
 
     def _set_job_printer_text (self, column, cell, model, iter, *data):
         jobid = model.get_value (iter, 0)
-        reasons = self.jobs[jobid].get('job-state-reasons')
+        try:
+            reasons = self.jobs[jobid].get('job-state-reasons')
+        except KeyError:
+            return
+
         if reasons == 'printer-stopped':
             reason = ' - ' + _("disabled")
         else:
@@ -2250,14 +2279,22 @@ class JobViewer (GtkGUI):
 
     def _set_job_size_text (self, column, cell, model, iter, *data):
         jobid = model.get_value (iter, 0)
-        job = self.jobs[jobid]
+        try:
+            job = self.jobs[jobid]
+        except KeyError:
+            return
+
         size = _("Unknown")
-        if job.has_key ('job-k-octets'):
+        if 'job-k-octets' in job:
             size = str (job['job-k-octets']) + 'k'
         cell.set_property("text", size)
 
     def _find_job_state_text (self, job):
-        data = self.jobs[job]
+        try:
+            data = self.jobs[job]
+        except KeyError:
+            return
+
         jstate = data.get ('job-state', cups.IPP_JOB_PROCESSING)
         s = int (jstate)
         job_requires_auth = (s == cups.IPP_JOB_HELD and
@@ -2337,7 +2374,11 @@ class JobViewer (GtkGUI):
 
     def _set_job_status_icon (self, column, cell, model, iter, *data):
         jobid = model.get_value (iter, 0)
-        data = self.jobs[jobid]
+        try:
+            data = self.jobs[jobid]
+        except KeyError:
+            return
+
         jstate = data.get ('job-state', cups.IPP_JOB_PROCESSING)
         s = int (jstate)
         if s == cups.IPP_JOB_PROCESSING:
@@ -2369,7 +2410,11 @@ class JobViewer (GtkGUI):
 
     def _set_job_status_text (self, column, cell, model, iter, *data):
         jobid = model.get_value (iter, 0)
-        data = self.jobs[jobid]
+        try:
+            data = self.jobs[jobid]
+        except KeyError:
+            return
+
         try:
             text = data['_status_text']
         except KeyError:
