@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 
 ## Copyright (C) 2010, 2011, 2012, 2013 Red Hat, Inc.
 ## Authors:
@@ -48,7 +48,7 @@ class PPDCache:
             self._cups.destroy ()
 
     def fetch_ppd (self, name, callback, check_uptodate=True):
-        if check_uptodate and self._modtimes.has_key (name):
+        if check_uptodate and name in self._modtimes:
             # We have getPPD3 so we can check whether the PPD is up to
             # date.
             debugprint ("%s: check if %s is up to date" % (self, name))
@@ -86,7 +86,7 @@ class PPDCache:
         # leave temporary files around even though we are caching...
         f.seek (0)
         (tmpfd, tmpfname) = tempfile.mkstemp ()
-        tmpf = file (tmpfname, "w")
+        tmpf = open (tmpfname, "w")
         tmpf.writelines (f.readlines ())
         del tmpf
         os.close (tmpfd)
@@ -111,7 +111,7 @@ class PPDCache:
         else:
             # Store an open file object, then remove the actual file.
             # This way we don't leave temporary files around.
-            self._cache[name] = file (result)
+            self._cache[name] = open (result)
             debugprint ("%s: caching %s (fd %d)" % (self, result,
                                                     self._cache[name].fileno()))
             os.unlink (result)
@@ -137,7 +137,7 @@ class PPDCache:
                 # file.  This way we don't leave temporary files
                 # around.
                 try:
-                    self._cache[name] = file (filename)
+                    self._cache[name] = open (filename)
                     debugprint ("%s: caching %s (fd %d) "
                                 "(%s) - %s" % (self, filename,
                                                self._cache[name].fileno (),
@@ -182,7 +182,6 @@ if __name__ == "__main__":
     from debug import *
     from gi.repository import GObject
     set_debugging (True)
-    GObject.threads_init ()
     Gdk.threads_init ()
     loop = GObject.MainLoop ()
 
