@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 
 ## system-config-printer
 
@@ -31,7 +31,7 @@ import asyncconn
 from debug import debugprint
 import config
 import gettext
-gettext.install(domain=config.PACKAGE, localedir=config.localedir, unicode=True)
+gettext.install(domain=config.PACKAGE, localedir=config.localedir)
 
 class PPDsLoader(GObject.GObject):
     """
@@ -85,11 +85,10 @@ class PPDsLoader(GObject.GObject):
 
         fmt = _("Searching")
         self._dialog = Gtk.MessageDialog (parent=parent,
-                                          flags=Gtk.DialogFlags.MODAL |
-                                          Gtk.DialogFlags.DESTROY_WITH_PARENT,
-                                          type=Gtk.MessageType.INFO,
+                                          modal=True, destroy_with_parent=True,
+                                          message_type=Gtk.MessageType.INFO,
                                           buttons=Gtk.ButtonsType.CANCEL,
-                                          message_format=fmt)
+                                          text=fmt)
 
         self._dialog.format_secondary_text (_("Searching for drivers"))
 
@@ -171,7 +170,7 @@ class PPDsLoader(GObject.GObject):
                                          self._device_make_and_model)
 
             ppdnamelist = ppds.\
-                orderPPDNamesByPreference (fit.keys (),
+                orderPPDNamesByPreference (list(fit.keys ()),
                                            self._installed_files,
                                            devid=self._devid_dict,
                                            fit=fit)
@@ -282,7 +281,7 @@ if __name__ == "__main__":
     class Foo:
         def __init__ (self):
             w = Gtk.Window ()
-            b = Gtk.Button ("Go")
+            b = Gtk.Button.new_with_label ("Go")
             w.add (b)
             b.connect ('clicked', self.go)
             w.connect ('delete-event', Gtk.main_quit)
@@ -299,15 +298,14 @@ if __name__ == "__main__":
             self._window.destroy ()
             Gtk.main_quit ()
             exc = ppdsloader.get_error ()
-            print exc
+            print(exc)
             ppds = ppdsloader.get_ppds ()
             if ppds != None:
-                print len (ppds)
+                print(len (ppds))
 
             ppdsloader.destroy ()
 
     from debug import set_debugging
     set_debugging (True)
-    GObject.threads_init ()
     Foo ()
     Gtk.main ()
