@@ -1,8 +1,8 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 
 ## system-config-printer
 
-## Copyright (C) 2010, 2011, 2012, 2013 Red Hat, Inc.
+## Copyright (C) 2010, 2011, 2012, 2013, 2014 Red Hat, Inc.
 ## Authors:
 ##  Tim Waugh <twaugh@redhat.com>
 
@@ -24,6 +24,7 @@ import dbus.service
 from gi.repository import GObject
 from gi.repository import GLib
 from gi.repository import Gdk
+from gi.repository import Gtk
 import sys
 
 from debug import *
@@ -194,36 +195,8 @@ class GetBestDriversRequest:
                                                           fit=fit)
             ppdname = ppdnamelist[0]
             status = fit[ppdname]
-
-            try:
-                if status != "exact" and not self.download_tried:
-                    self.download_tried = True
-                    self.loop = GObject.MainLoop ()
-                    self.dialog = newprinter.NewPrinterGUI()
-                    self.dialog.NewPrinterWindow.set_modal (False)
-                    self.handles = \
-                        [self.dialog.connect ('dialog-canceled',
-                                              self.on_dialog_canceled),
-                         self.dialog.connect ('driver-download-checked',
-                                              self.on_driver_download_checked)]
-                    if self.dialog.init ('download_driver',
-                                         devid=self.device_id):
-                        self.loop.run()
-                    for handle in self.handles:
-                        self.dialog.disconnect (handle)
-                    if len(self.installed_files) > 0:
-                        debugprint ("GetBestDrivers request: Re-fetch PPDs after driver download")
-                        g_ppds = FetchedPPDs (self.cupsconn, self.language)
-                        self._signals.append (g_ppds.connect ('ready', self._ppds_ready))
-                        self._signals.append (g_ppds.connect ('error', self._ppds_error))
-                        g_ppds.run ()
-                        return
-            except:
-                # Ignore driver download if packages needed for the GUI are not
-                # installed or if no windows can be opened
-                pass
             g_killtimer.remove_hold ()
-            self.reply_handler (map (lambda x: (x, fit[x]), ppdnamelist))
+            self.reply_handler ([(x, fit[x]) for x in ppdnamelist])
         except Exception as e:
             try:
                 g_killtimer.remove_hold ()
@@ -251,7 +224,7 @@ class GroupPhysicalDevicesRequest:
             g_killtimer.add_hold ()
             need_resolving = {}
             self.deviceobjs = {}
-            for device_uri, device_dict in self.devices.iteritems ():
+            for device_uri, device_dict in self.devices.items ():
                 deviceobj = cupshelpers.Device (device_uri, **device_dict)
                 self.deviceobjs[device_uri] = deviceobj
                 if device_uri.startswith ("dnssd://"):
@@ -285,8 +258,7 @@ class GroupPhysicalDevicesRequest:
 
             uris_by_phys = []
             for physdev in self.physdevs:
-                uris_by_phys.append (map (lambda x: x.uri,
-                                          physdev.get_devices ()))
+                uris_by_phys.append ([x.uri for x in physdev.get_devices ()])
 
             g_killtimer.remove_hold ()
             self.reply_handler (uris_by_phys)
@@ -436,6 +408,7 @@ class ConfigPrintingJobApplet(dbus.service.Object):
         Gdk.threads_enter ()
         self.jobapplet = jobviewer.JobViewer(bus=dbus.SystemBus (),
                                              applet=True, my_jobs=True)
+        self.jobapplet.set_process_pending (False)
         Gdk.threads_leave ()
         handle = self.jobapplet.connect ('finished', self.on_jobapplet_finished)
         self.finished_handle = handle
@@ -538,7 +511,7 @@ def _client_demo ():
             sys.argv[3] == '--devid'):
             device_id = sys.argv[4]
     else:
-        print "Device URI required"
+        print ("Device URI required")
         return
 
     from gi.repository import Gtk
@@ -552,11 +525,11 @@ def _client_demo ():
     iface = dbus.Interface (obj, CONFIG_NEWPRINTERDIALOG_IFACE)
     loop = GObject.MainLoop ()
     def on_canceled(path=None):
-        print "%s: Dialog canceled" % path
+        print ("%s: Dialog canceled" % path)
         loop.quit ()
 
     def on_added(name, path=None):
-        print "%s: Printer '%s' added" % (path, name)
+        print ("%s: Printer '%s' added" % (path, name))
         loop.quit ()
 
     iface.connect_to_signal ("DialogCanceled", on_canceled,
@@ -571,7 +544,7 @@ if __name__ == '__main__':
     import ppdippstr
     import config
     import gettext
-    gettext.install(domain=config.PACKAGE, localedir=config.localedir, unicode=True)
+    gettext.install(domain=config.PACKAGE, localedir=config.localedir)
 
     import locale
     try:
@@ -597,8 +570,9 @@ if __name__ == '__main__':
         sys.exit (0)
 
     debugprint ("Service running...")
-    loop = GObject.MainLoop ()
-    g_killtimer = KillTimer (killfunc=loop.quit)
+    g_killtimer = KillTimer (killfunc=Gtk.main_quit)
     cp = ConfigPrinting ()
-    loop.run ()
+    Gdk.threads_enter ()
+    Gtk.main ()
+    Gdk.threads_leave ()
     cp.destroy ()
