@@ -1,6 +1,6 @@
 /* -*- Mode: C; c-file-style: "gnu" -*-
  * udev-configure-printer - a udev callout to configure print queues
- * Copyright (C) 2009, 2010, 2011 Red Hat, Inc.
+ * Copyright (C) 2009, 2010, 2011, 2012, 2014 Red Hat, Inc.
  * Author: Tim Waugh <twaugh@redhat.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,10 +22,13 @@
 /*
  * The protocol for this program is:
  *
- * udev-configure-printer add {DEVPATH}
- * udev-configure-printer remove {DEVPATH}
+ * udev-configure-printer add {DEVADDR}
+ * udev-configure-printer remove {DEVADDR}
  *
- * where DEVPATH is the path (%p) of the device
+ * where DEVADDR is one of:
+ *   the USB address of the device in the form usb-$env{BUSNUM}-$env{DEVNUM}
+ *   the device path of the device (%p)
+ *   the bluetooth address of the device
  */
 
 #define LIBUDEV_I_KNOW_THE_API_IS_SUBJECT_TO_CHANGE 1
@@ -524,22 +527,66 @@ device_file_filter(const struct dirent *entry)
 }
 
 static char *
-device_id_from_devpath (const char *devpath,
-			const struct usb_uri_map *map,
-			struct device_id *id,
-			char *usbserial, size_t usbseriallen,
-			char *usblpdev, size_t usblpdevlen)
+get_ieee1284_id_from_child (struct udev *udev, struct udev_device *parent)
 {
-  struct usb_uri_map_entry *entry;
-  struct udev *udev;
-  struct udev_device *dev, *parent_dev = NULL;
-  const char *idVendorStr, *idProductStr, *serial;
-  char *end;
+  struct udev_enumerate *udev_enum;
+  struct udev_list_entry *item, *first = NULL;
+  char *device_id = NULL;
+
+  udev_enum = udev_enumerate_new (udev);
+  if (!udev_enum)
+    {
+      syslog (LOG_ERR, "udev_enumerate_new failed");
+      exit (1);
+    }
+
+  if (udev_enumerate_add_match_parent (udev_enum, parent) < 0)
+    {
+      udev_enumerate_unref (udev_enum);
+      syslog (LOG_ERR, "uname to add parent match");
+      exit (1);
+    }
+
+  if (udev_enumerate_scan_devices (udev_enum) < 0)
+    {
+      udev_enumerate_unref (udev_enum);
+      syslog (LOG_ERR, "udev_enumerate_scan_devices failed");
+      exit (1);
+    }
+
+  first = udev_enumerate_get_list_entry (udev_enum);
+  udev_list_entry_foreach (item, first)
+    {
+      const char *ieee1284_id = NULL;
+      struct udev_device *dev;
+      dev = udev_device_new_from_syspath (udev,
+					  udev_list_entry_get_name (item));
+      if (dev == NULL)
+	continue;
+
+      ieee1284_id = udev_device_get_sysattr_value (dev, "ieee1284_id");
+      if (ieee1284_id)
+	device_id = g_strdup (ieee1284_id);
+
+      udev_device_unref (dev);
+      if (device_id)
+	break;
+    }
+
+  udev_enumerate_unref (udev_enum);
+  return device_id;
+}
+
+static char *
+get_ieee1284_id_using_libusb (struct udev_device *dev,
+			      const char *usbserial)
+{
+  const char *idVendorStr, *idProductStr;
   unsigned long idVendor, idProduct;
-  size_t syslen, devpathlen;
-  char *syspath, *devicefilepath;
+  char *end;
+  int conf = 0, iface = 0, altset = 0, numdevs = 0, i, n, m;
   libusb_device **list;
-  struct libusb_device	*device;
+  struct libusb_device *device;
   struct libusb_device_handle *handle = NULL;
   struct libusb_device_descriptor devdesc;
   struct libusb_config_descriptor *confptr = NULL;
@@ -547,140 +594,13 @@ device_id_from_devpath (const char *devpath,
   const struct libusb_interface_descriptor *altptr = NULL;
   char libusbserial[1024];
   char ieee1284_id[1024];
-  const char *device_id = NULL;
-  int conf = 0, iface = 0, altset = 0, numdevs = 0, i, n, m;
   int got = 0;
-  char *usb_device_devpath;
-  char *usblpdevpos, *dest;
-  struct dirent **namelist;
-  int num_names;
 
-  id->full_device_id = id->mfg = id->mdl = id->sern = NULL;
-
-  udev = udev_new ();
-  if (udev == NULL)
-    {
-      syslog (LOG_ERR, "udev_new failed");
-      exit (1);
-    }
-
-  syslen = strlen ("/sys");
-  devpathlen = strlen (devpath);
-  syspath = malloc (syslen + devpathlen + 1);
-  if (syspath == NULL)
-    {
-      udev_unref (udev);
-      syslog (LOG_ERR, "out of memory");
-      exit (1);
-    }
-  memcpy (syspath, "/sys", syslen);
-  memcpy (syspath + syslen, devpath, devpathlen);
-  syspath[syslen + devpathlen] = '\0';
-
-  devicefilepath = malloc (syslen + devpathlen + 5);
-  if (devicefilepath == NULL)
-    {
-      udev_unref (udev);
-      syslog (LOG_ERR, "out of memory");
-      exit (1);
-    }
-  memcpy (devicefilepath, syspath, syslen + devpathlen);
-  memcpy (devicefilepath + syslen + devpathlen, "/usb", 4);
-  devicefilepath[syslen + devpathlen + 4] = '\0';
-
-  /* For devices under control of the usblp kernel module we read out the number
-   * of the /dev/usb/lp* device file, as there can be queues set up with 
-   * non-standard CUPS backends based on the /dev/usb/lp* device file and
-   * we want to avoid that an additional queue with a standard CUPS backend
-   * gets set up.
-   */
-  num_names = scandir(devicefilepath, &namelist, device_file_filter, alphasort);
-  if (num_names <= 0)
-    num_names = scandir(syspath, &namelist, device_file_filter, alphasort);
-  if (num_names > 0)
-    {
-      usblpdevpos = strstr(namelist[0]->d_name, "lp");
-      if (usblpdevpos != NULL)
-	{
-	  usblpdevpos += 2;
-	  for (dest = usblpdev;
-	       (*usblpdevpos >= '0') && (*usblpdevpos <= '9') &&
-		 (dest - usblpdev < usblpdevlen);
-	       usblpdevpos ++, dest ++)
-	    *dest = *usblpdevpos;
-	  *dest = '\0';
-	}
-    }
-
-  dev = udev_device_new_from_syspath (udev, syspath);
-  if (dev == NULL)
-    {
-      udev_device_unref (dev);
-      udev_unref (udev);
-      syslog (LOG_ERR, "unable to access %s", syspath);
-      return NULL;
-    }
-
-  parent_dev = udev_device_get_parent_with_subsystem_devtype (dev,
-							      "usb",
-							      "usb_device");
-  if (!parent_dev)
-    {
-      udev_unref (udev);
-      syslog (LOG_ERR, "Failed to get parent");
-      return NULL;
-    }
-
-  usb_device_devpath = strdup (udev_device_get_devpath (parent_dev));
-  syslog (LOG_DEBUG, "device devpath is %s", usb_device_devpath);
-
-  for (entry = map->entries; entry; entry = entry->next)
-    if (!strcmp (entry->devpath, usb_device_devpath))
-      break;
-
-  if (entry)
-    {
-      /* The map already had an entry so has already been dealt
-       * with.  This can happen because there are two "add"
-       * triggers: one for the usb_device device and the other for
-       * the usblp device.  We have most likely been triggered by
-       * the usblp device, so the usb_device rule got there before
-       * us and succeeded.
-       *
-       * Pretend we didn't find any device URIs that matched, and
-       * exit.
-       */
-      syslog (LOG_DEBUG, "Device already handled");
-      return NULL;
-    }
-
-  serial = udev_device_get_sysattr_value (parent_dev, "serial");
-  if (serial)
-    {
-      strncpy (usbserial, serial, usbseriallen);
-      usbserial[usbseriallen - 1] = '\0';
-    }
-  else
-    usbserial[0] = '\0';
-
-  /* See if  the device is controlled by the usblp kernel module
-     and so the module has already read the device ID for us */
-  device_id = udev_device_get_sysattr_value (dev, "ieee1284_id");
-  if (device_id)
-    {
-      got = 1;
-      goto got_deviceid;
-    }
-
-  /* The usblp kernel module is not attached to this device. Use libusb to
-     fetch the Device ID. */
-  idVendorStr = udev_device_get_sysattr_value (parent_dev, "idVendor");
-  idProductStr = udev_device_get_sysattr_value (parent_dev, "idProduct");
+  idVendorStr = udev_device_get_sysattr_value (dev, "idVendor");
+  idProductStr = udev_device_get_sysattr_value (dev, "idProduct");
 
   if (!idVendorStr || !idProductStr)
     {
-      udev_device_unref (dev);
-      udev_unref (udev);
       syslog (LOG_ERR, "Missing sysattr %s",
 	      idVendorStr ?
 	      (idProductStr ? "serial" : "idProduct") : "idVendor");
@@ -806,16 +726,126 @@ device_id_from_devpath (const char *devpath,
   libusb_free_device_list(list, 1);
   libusb_exit(NULL);
 
- got_deviceid:
   if (got)
+    return g_strdup (ieee1284_id + 2);
+  return NULL;
+}
+
+static char *
+device_id_from_devpath (struct udev *udev, const char *devpath,
+			const struct usb_uri_map *map,
+			struct device_id *id,
+			char *usbserial, size_t usbseriallen,
+			char *usblpdev, size_t usblpdevlen)
+{
+  struct usb_uri_map_entry *entry;
+  struct udev_device *dev;
+  const char *serial;
+  size_t syslen, devpathlen;
+  char *syspath, *devicefilepath;
+  const char *device_id = NULL;
+  char *usb_device_devpath;
+  char *usblpdevpos, *dest;
+  struct dirent **namelist;
+  int num_names;
+
+  id->full_device_id = id->mfg = id->mdl = id->sern = NULL;
+
+  syslen = strlen ("/sys");
+  devpathlen = strlen (devpath);
+  syspath = malloc (syslen + devpathlen + 1);
+  if (syspath == NULL)
     {
-      if (!device_id)
-	device_id = ieee1284_id + 2;
-      parse_device_id (device_id, id);
+      syslog (LOG_ERR, "out of memory");
+      exit (1);
+    }
+  memcpy (syspath, "/sys", syslen);
+  memcpy (syspath + syslen, devpath, devpathlen);
+  syspath[syslen + devpathlen] = '\0';
+
+  devicefilepath = malloc (syslen + devpathlen + 5);
+  if (devicefilepath == NULL)
+    {
+      syslog (LOG_ERR, "out of memory");
+      exit (1);
+    }
+  memcpy (devicefilepath, syspath, syslen + devpathlen);
+  memcpy (devicefilepath + syslen + devpathlen, "/usb", 4);
+  devicefilepath[syslen + devpathlen + 4] = '\0';
+
+  /* For devices under control of the usblp kernel module we read out the number
+   * of the /dev/usb/lp* device file, as there can be queues set up with 
+   * non-standard CUPS backends based on the /dev/usb/lp* device file and
+   * we want to avoid that an additional queue with a standard CUPS backend
+   * gets set up.
+   */
+  num_names = scandir(devicefilepath, &namelist, device_file_filter, alphasort);
+  if (num_names <= 0)
+    num_names = scandir(syspath, &namelist, device_file_filter, alphasort);
+  if (num_names > 0)
+    {
+      usblpdevpos = strstr(namelist[0]->d_name, "lp");
+      if (usblpdevpos != NULL)
+	{
+	  usblpdevpos += 2;
+	  for (dest = usblpdev;
+	       (*usblpdevpos >= '0') && (*usblpdevpos <= '9') &&
+		 (dest - usblpdev < usblpdevlen);
+	       usblpdevpos ++, dest ++)
+	    *dest = *usblpdevpos;
+	  *dest = '\0';
+	}
     }
 
+  dev = udev_device_new_from_syspath (udev, syspath);
+  if (dev == NULL)
+    {
+      udev_device_unref (dev);
+      syslog (LOG_ERR, "unable to access %s", syspath);
+      return NULL;
+    }
+
+  usb_device_devpath = strdup (udev_device_get_devpath (dev));
+  syslog (LOG_DEBUG, "device devpath is %s", usb_device_devpath);
+
+  for (entry = map->entries; entry; entry = entry->next)
+    if (!strcmp (entry->devpath, usb_device_devpath))
+      break;
+
+  if (entry)
+    {
+      /* The map already had an entry so has already been dealt
+       * with.  This can happen because there are two "add"
+       * triggers: one for the usb_device device and the other for
+       * the usblp device.  We have most likely been triggered by
+       * the usblp device, so the usb_device rule got there before
+       * us and succeeded.
+       *
+       * Pretend we didn't find any device URIs that matched, and
+       * exit.
+       */
+      syslog (LOG_DEBUG, "Device already handled");
+      return NULL;
+    }
+
+  serial = udev_device_get_sysattr_value (dev, "serial");
+  if (serial)
+    {
+      strncpy (usbserial, serial, usbseriallen);
+      usbserial[usbseriallen - 1] = '\0';
+    }
+  else
+    usbserial[0] = '\0';
+
+  device_id = get_ieee1284_id_from_child (udev, dev);
+  if (!device_id)
+    /* Use libusb to fetch the Device ID. */
+    device_id = get_ieee1284_id_using_libusb (dev, usbserial);
+
+  if (device_id)
+    parse_device_id (device_id, id);
+
   udev_device_unref (dev);
-  udev_unref (udev);
   return usb_device_devpath;
 }
 
@@ -849,6 +879,70 @@ device_id_from_bluetooth (const char *bdaddr, struct device_id *id)
   g_free (device_id);
 }
 
+static char *
+devpath_from_usb_devaddr (struct udev *udev, const char *devaddr)
+{
+  char *devname_ending = g_strdup (devaddr);
+  char *devname;
+  const char *devpath;
+  struct udev_enumerate *udev_enum;
+  struct udev_list_entry *first = NULL;
+  struct udev_device *device;
+
+  g_strdelimit (devname_ending, "-", '/');
+  devname = g_strdup_printf("/dev/bus/%s", devname_ending);
+  g_free (devname_ending);
+
+  udev_enum = udev_enumerate_new (udev);
+  if (udev_enum == NULL)
+    {
+      syslog (LOG_ERR, "udev_enumerate_new failed");
+      exit (1);
+    }
+
+  if (udev_enumerate_add_match_property (udev_enum, "DEVNAME", devname) < 0)
+    {
+      udev_enumerate_unref (udev_enum);
+      syslog (LOG_ERR, "udev_enumerate_add_match_property failed");
+      exit (1);
+    }
+
+  if (udev_enumerate_scan_devices (udev_enum) < 0)
+    {
+      udev_enumerate_unref (udev_enum);
+      syslog (LOG_ERR, "udev_enumerate_scan_devices failed");
+      exit (1);
+    }
+
+  first = udev_enumerate_get_list_entry (udev_enum);
+  if (first == NULL)
+    {
+      udev_enumerate_unref (udev_enum);
+      syslog (LOG_ERR, "no device named %s found", devname);
+      exit (1);
+    }
+
+  device = udev_device_new_from_syspath (udev,
+					 udev_list_entry_get_name (first));
+  if (device == NULL)
+    {
+      udev_enumerate_unref (udev_enum);
+      syslog (LOG_ERR, "unable to examine device");
+      exit (1);
+    }
+
+  devpath = udev_device_get_devpath (device);
+  udev_enumerate_unref (udev_enum);
+  if (!devpath)
+    {
+      syslog (LOG_ERR, "no devpath for device");
+      exit (1);
+    }
+
+  g_free (devname);
+  return g_strdup (devpath);
+}
+
 static char *
 uri_from_bdaddr (const char *devpath)
 {
@@ -1490,52 +1584,44 @@ bluetooth_verify_address (const char *bdaddr)
 }
 
 static int
-do_add (const char *cmd, const char *devpath, int detach)
+do_add (const char *cmd, const char *devaddr)
 {
-  pid_t pid;
-  int f;
   struct device_id id;
   struct device_uris device_uris;
   struct usb_uri_map *map;
+  struct udev *udev;
+  char *devpath = NULL;
   char *usb_device_devpath = NULL;
   char usbserial[256];
   char usblpdev[8] = "";
   gboolean is_bluetooth;
 
-  if (detach && getenv ("DEBUG") == NULL)
-    {
-      if ((pid = fork ()) == -1)
-	syslog (LOG_ERR, "Failed to fork process");
-      else if (pid != 0)
-	/* Parent. */
-	exit (0);
+  syslog (LOG_DEBUG, "add %s", devaddr);
 
-      close (STDIN_FILENO);
-      close (STDOUT_FILENO);
-      close (STDERR_FILENO);
-      f = open ("/dev/null", O_RDWR);
-      if (f != STDIN_FILENO)
-	dup2 (f, STDIN_FILENO);
-      if (f != STDOUT_FILENO)
-	dup2 (f, STDOUT_FILENO);
-      if (f != STDERR_FILENO)
-	dup2 (f, STDERR_FILENO);
-
-      setsid ();
-    }
-
-  syslog (LOG_DEBUG, "add %s", devpath);
-
-  is_bluetooth = bluetooth_verify_address (devpath);
+  is_bluetooth = bluetooth_verify_address (devaddr);
 
   map = read_usb_uri_map ();
   if (is_bluetooth) {
     usbserial[0] = '\0';
-    device_id_from_bluetooth (devpath, &id);
+    device_id_from_bluetooth (devaddr, &id);
   } else {
-    usb_device_devpath = device_id_from_devpath (devpath, map, &id,
+    udev = udev_new ();
+    if (udev == NULL)
+      {
+	syslog (LOG_ERR, "udev_new failed");
+	exit (1);
+      }
+
+    if (!strncmp (devaddr, "usb-", 4))
+      devpath = devpath_from_usb_devaddr (udev, devaddr);
+    else
+      devpath = g_strdup (devaddr);
+
+    usb_device_devpath = device_id_from_devpath (udev, devpath, map, &id,
 						 usbserial, sizeof (usbserial),
 						 usblpdev, sizeof (usblpdev));
+    g_free (devpath);
+    udev_unref (udev);
   }
 
   if (!id.mfg || !id.mdl)
@@ -1597,12 +1683,14 @@ do_add (const char *cmd, const char *devpath, int detach)
       argv[i + 2] = NULL;
 
       syslog (LOG_DEBUG, "About to add queue for %s", argv[2]);
-      strcpy (argv0, cmd);
+      strncpy (argv0, cmd, sizeof (argv0));
+      argv0[sizeof (argv0) - 1] = '\0';
       p = strrchr (argv0, '/');
       if (p++ == NULL)
 	p = argv0;
 
-      strcpy (p, "udev-add-printer");
+      strncpy (p, "udev-add-printer", sizeof (argv0) - (p - argv0));
+      argv0[sizeof (argv0) - 1] = '\0';
 
       execv (argv0, argv);
       syslog (LOG_ERR, "Failed to execute %s", argv0);
@@ -1616,7 +1704,7 @@ do_add (const char *cmd, const char *devpath, int detach)
 static void
 remove_queue (const char *printer_uri)
 {
-  /* Disable it. */
+  /* Delete it. */
   http_t *cups = httpConnectEncrypt (cupsServer (), ippPort (),
 				     cupsEncryption ());
   ipp_t *request, *answer;
@@ -1680,24 +1768,40 @@ disable_queue (const char *printer_uri, void *context)
 }
 
 static int
-do_remove (const char *devpath)
+do_remove (const char *devaddr)
 {
   struct usb_uri_map *map;
   struct usb_uri_map_entry *entry, **prev;
   struct device_uris *uris = NULL;
   char usblpdev[8] = "";
-  syslog (LOG_DEBUG, "remove %s", devpath);
+  gchar *devpath = NULL;
+  syslog (LOG_DEBUG, "remove %s", devaddr);
 
-  if (bluetooth_verify_address (devpath))
+  if (bluetooth_verify_address (devaddr))
     {
       char *device_uri;
 
-      device_uri = uri_from_bdaddr (devpath);
-      remove_queue (devpath);
+      device_uri = uri_from_bdaddr (devaddr);
+      remove_queue (device_uri);
       g_free (device_uri);
       return 0;
     }
 
+  if (!strncmp (devaddr, "usb-", 4))
+    {
+      struct udev *udev = udev_new ();
+      if (udev == NULL)
+	{
+	  syslog (LOG_ERR, "udev_new failed");
+	  exit (1);
+	}
+
+      devpath = devpath_from_usb_devaddr (udev, devaddr);
+      udev_unref (udev);
+    }
+  else
+    devpath = g_strdup (devaddr);
+
   map = read_usb_uri_map ();
   prev = &map->entries;
   for (entry = map->entries; entry; entry = entry->next)
@@ -1721,76 +1825,10 @@ do_remove (const char *devpath)
     }
 
   free_usb_uri_map (map);
+  g_free (devpath);
   return 0;
 }
 
-static int
-do_enumerate (const char *argv0)
-{
-  struct udev *udev;
-  struct udev_enumerate *e;
-  struct udev_list_entry *item = NULL, *first = NULL;
-  int r = 0;
-
-  udev = udev_new ();
-  if (udev == NULL)
-    {
-      syslog (LOG_ERR, "udev_new failed");
-      exit (1);
-    }
-
-  e = udev_enumerate_new (udev);
-  if (e == NULL)
-    {
-      udev_unref (udev);
-      syslog (LOG_ERR, "udev_enumerate_new failed");
-      exit (1);
-    }
-
-  if (udev_enumerate_add_match_tag (e, "udev-configure-printer") < 0)
-    {
-      udev_unref (udev);
-      udev_enumerate_unref (e);
-      syslog (LOG_ERR, "udev_enumerate_add_match_tag failed");
-      exit (1);
-    }
-
-  if (udev_enumerate_scan_devices (e) < 0)
-    {
-      udev_unref (udev);
-      udev_enumerate_unref (e);
-      syslog (LOG_ERR, "udev_enumerate_scan_devices failed");
-      exit (1);
-    }
-
-  first = udev_enumerate_get_list_entry (e);
-  udev_list_entry_foreach (item, first)
-    {
-      int k;
-      const char *p;
-      struct udev_device *d;
-
-      d = udev_device_new_from_syspath (udev, udev_list_entry_get_name(item));
-      if (!d)
-        continue;
-
-      p = udev_device_get_devpath (d);
-      if (p)
-        {
-          k = do_add (argv0, p, 0);
-          if (k != 0)
-            r = k;
-        }
-
-      udev_device_unref (d);
-  }
-
-  udev_enumerate_unref (e);
-  udev_unref (udev);
-
-  return r;
-}
-
 int
 main (int argc, char **argv)
 {
@@ -1818,9 +1856,9 @@ main (int argc, char **argv)
   openlog ("udev-configure-printer", 0, LOG_LPR);
   cupsSetPasswordCB (no_password);
   if (add)
-    return do_add (argv[0], argv[2], 1);
+    return do_add (argv[0], argv[2]);
   if (enumerate)
-    return do_enumerate (argv[0]);
+    return 0; // no-op
 
   return do_remove (argv[2]);
 }
